# Generated by roxygen2: do not edit by hand

S3method(slider_minus,slider_test_class.double)
S3method(slider_plus,slider_test_class.double)
export(block)
export(hop)
export(hop2)
export(hop2_vec)
export(hop_index)
export(hop_index2)
export(hop_index2_vec)
export(hop_index_vec)
export(hop_vec)
export(phop)
export(phop_index)
export(phop_index_vec)
export(phop_vec)
export(pslide)
export(pslide_chr)
export(pslide_dbl)
export(pslide_dfc)
export(pslide_dfr)
export(pslide_index)
export(pslide_index_chr)
export(pslide_index_dbl)
export(pslide_index_dfc)
export(pslide_index_dfr)
export(pslide_index_int)
export(pslide_index_lgl)
export(pslide_index_vec)
export(pslide_int)
export(pslide_lgl)
export(pslide_period)
export(pslide_period_chr)
export(pslide_period_dbl)
export(pslide_period_dfc)
export(pslide_period_dfr)
export(pslide_period_int)
export(pslide_period_lgl)
export(pslide_period_vec)
export(pslide_vec)
export(slide)
export(slide2)
export(slide2_chr)
export(slide2_dbl)
export(slide2_dfc)
export(slide2_dfr)
export(slide2_int)
export(slide2_lgl)
export(slide2_vec)
export(slide_all)
export(slide_any)
export(slide_chr)
export(slide_dbl)
export(slide_dfc)
export(slide_dfr)
export(slide_index)
export(slide_index2)
export(slide_index2_chr)
export(slide_index2_dbl)
export(slide_index2_dfc)
export(slide_index2_dfr)
export(slide_index2_int)
export(slide_index2_lgl)
export(slide_index2_vec)
export(slide_index_all)
export(slide_index_any)
export(slide_index_chr)
export(slide_index_dbl)
export(slide_index_dfc)
export(slide_index_dfr)
export(slide_index_int)
export(slide_index_lgl)
export(slide_index_max)
export(slide_index_mean)
export(slide_index_min)
export(slide_index_prod)
export(slide_index_sum)
export(slide_index_vec)
export(slide_int)
export(slide_lgl)
export(slide_max)
export(slide_mean)
export(slide_min)
export(slide_period)
export(slide_period2)
export(slide_period2_chr)
export(slide_period2_dbl)
export(slide_period2_dfc)
export(slide_period2_dfr)
export(slide_period2_int)
export(slide_period2_lgl)
export(slide_period2_vec)
export(slide_period_chr)
export(slide_period_dbl)
export(slide_period_dfc)
export(slide_period_dfr)
export(slide_period_int)
export(slide_period_lgl)
export(slide_period_vec)
export(slide_prod)
export(slide_sum)
export(slide_vec)
export(slider_minus)
export(slider_plus)
import(rlang)
import(vctrs)
importFrom(warp,warp_boundary)
importFrom(warp,warp_distance)
useDynLib(slider, .registration = TRUE)
